import 'package:flutter/material.dart';
import 'dart:typed_data';
import 'dart:convert';
import 'package:pointycastle/export.dart' as crypto;
import 'package:file_picker/file_picker.dart';
import 'dart:io';
import 'key_storage.dart';

class ExportPrivateKeyPage extends StatefulWidget {
  const ExportPrivateKeyPage({super.key});

  @override
  _ExportPrivateKeyPageState createState() => _ExportPrivateKeyPageState();
}

class _ExportPrivateKeyPageState extends State<ExportPrivateKeyPage> {
  final TextEditingController _passwordController = TextEditingController();

  Future<void> _exportPrivateKey() async {
    final keyStorage = KeyStorage();
    final privateKeyPem = await keyStorage.getPrivateKey();

    if (privateKeyPem == null) {
      // Show error message if there's no key
      ScaffoldMessenger.of(context).showSnackBar(
        const SnackBar(
          content: Text('No private key found to export.'),
        ),
      );
      return;
    }

    final password = _passwordController.text;
    if (password.isEmpty) {
      ScaffoldMessenger.of(context).showSnackBar(
        const SnackBar(
          content: Text('Please enter a password.'),
        ),
      );
      return;
    }

    final encryptedData = _encryptPrivateKey(privateKeyPem, password);

    final outputFile = await FilePicker.platform.saveFile(
      dialogTitle: 'Save encrypted private key',
      fileName: 'private_key_encrypted.aes',
    );

    if (outputFile != null) {
      try {
        final file = File(outputFile);
        await file.writeAsBytes(encryptedData);

        showDialog(
          context: context,
          builder: (context) => AlertDialog(
            title: const Text('Key Exported'),
            content: const Text('The encrypted private key has been exported successfully.'),
            actions: [
              TextButton(
                onPressed: () => Navigator.pop(context),
                child: const Text('OK'),
              ),
            ],
          ),
        );
      } catch (e) {
        ScaffoldMessenger.of(context).showSnackBar(
          SnackBar(
            content: Text('Failed to write file: $e'),
          ),
        );
      }
    }
  }

  Uint8List _encryptPrivateKey(String privateKey, String password) {
    // Derive a key from the password using PBKDF2
    final derivator = crypto.PBKDF2KeyDerivator(
      crypto.HMac(crypto.SHA256Digest(), 64),
    );

    final salt = Uint8List.fromList(utf8.encode('some_salt')); // In production, use a random salt and store it securely
    derivator.init(crypto.Pbkdf2Parameters(salt, 1000, 32));
    final key = derivator.process(Uint8List.fromList(utf8.encode(password)));

    // Initialize AES-CBC cipher with PKCS7 padding
    final iv = Uint8List(16); // zero IV for example, in production use random IV and store it
    final params = crypto.PaddedBlockCipherParameters<crypto.ParametersWithIV<crypto.KeyParameter>, Null>(
      crypto.ParametersWithIV<crypto.KeyParameter>(crypto.KeyParameter(key), iv),
      null,
    );

    final cipher = crypto.PaddedBlockCipher('AES/CBC/PKCS7');
    cipher.init(true, params);

    final input = Uint8List.fromList(utf8.encode(privateKey));
    final output = cipher.process(input);

    return output;
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.black,
      appBar: AppBar(
        title: const Text('Export Private Key'),
      ),
      body: Padding(
        padding: const EdgeInsets.all(16.0),
        child: Column(
          children: [
            const Text(
              'Enter a password to encrypt the private key:',
              style: TextStyle(color: Colors.white),
            ),
            TextField(
              controller: _passwordController,
              obscureText: true,
              style: const TextStyle(color: Colors.white),
              decoration: const InputDecoration(
                hintText: 'Password',
                hintStyle: TextStyle(color: Colors.grey),
              ),
            ),
            const SizedBox(height: 20),
            ElevatedButton(
              onPressed: _exportPrivateKey,
              child: const Text('Export Encrypted Private Key'),
            ),
          ],
        ),
      ),
    );
  }
}
